@@ -26,7 +26,8 @@ from torch.cuda.amp import GradScaler
 from torch.utils.data import DataLoader, Dataset
 
 from dstk.pytorch import (check_tensor, CallbackInterface, CallbackHandler,
-                          FitState, FitControl, DEFAULT_CALLBACKS)
+                          FitState, FitControl, is_dist_avail_and_initialized,
+                          DEFAULT_CALLBACKS)
 from dstk.utils.errors import DivergenceError
 
 
@@ -349,7 +350,7 @@ class BaseEnvironnement(BaseEstimator):
         loss_kargs: Dict[str, Any],
         state : FitState,
         control: FitControl,
-        list_callbacks: List[CallbackInterface]
+        list_callbacks: Optional[List[CallbackInterface]] = None
     ):
         r"""
         Méthode privée d'une étape de backpropagation du gradient.
@@ -369,7 +370,7 @@ class BaseEnvironnement(BaseEstimator):
         control : FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        list_callbacks : List[CallbackInterface]
+        list_callbacks : Optional[List[CallbackInterface]]
             Liste des callback permettant de contôler l'apprentissage durant 
             la phase d'évaluation.
 
@@ -377,7 +378,8 @@ class BaseEnvironnement(BaseEstimator):
         -------
         loss : Valeur de la fonction coût au travers du mini-batch.
         """
-        list_callbacks.begin_step(self, data, state, control, None)
+        if list_callbacks:
+            list_callbacks.begin_step(self, data, state, control, None)
         self.train()
         if control.device.type != "cpu":
             for key, value in data.items():
@@ -408,7 +410,8 @@ class BaseEnvironnement(BaseEstimator):
             self.optimizer.zero_grad()
         loss = loss_.detach()
         self._divergence(loss)
-        list_callbacks.end_step(self, data, state, control, loss)
+        if list_callbacks:
+            list_callbacks.end_step(self, data, state, control, loss)
         return loss
 
     def _eval_step(
@@ -463,7 +466,7 @@ class BaseEnvironnement(BaseEstimator):
         loss_kargs: Optional[Dict[str, Any]],
         state: FitState,
         control: FitControl,
-        list_callbacks: List[CallbackInterface]
+        list_callbacks: Optional[List[CallbackInterface]] = None
     ):
         r"""
         Méthode interne exécutant la phase d'évaluation durant l'apprentissage.
@@ -492,17 +495,18 @@ class BaseEnvironnement(BaseEstimator):
         control : FitControl
             Contrôle de l'apprentissage (instancé dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        list_callbacks : List[CallbackInterface]
+        list_callbacks : Optional[List[CallbackInterface]]
             Liste des callback permettant de contôler l'apprentissage durant 
             la phase d'évaluation.
         """
-        list_callbacks.begin_eval(
-            model=self,
-            data=eval_dataset,
-            state=state,
-            control=control,
-            res_loss=None
-        )
+        if list_callbacks:
+            list_callbacks.begin_eval(
+                model=self,
+                data=eval_dataset,
+                state=state,
+                control=control,
+                res_loss=None
+            )
         loss_eval = 0
         for jj, eval_batch in enumerate(eval_dataset):
             list_callbacks.begin_eval_step(
@@ -523,13 +527,14 @@ class BaseEnvironnement(BaseEstimator):
                 control.diverge = True
                 warn(error.msg)
                 break
-        list_callbacks.end_eval(
-            model=self,
-            data=eval_dataset,
-            state=state,
-            control=control,
-            res_loss=loss_eval/(jj+1)
-        )
+        if list_callbacks:
+            list_callbacks.end_eval(
+                model=self,
+                data=eval_dataset,
+                state=state,
+                control=control,
+                res_loss=loss_eval/(jj+1)
+            )
 
     def fit(
         self,
@@ -842,6 +847,11 @@ class BaseEnvironnement(BaseEstimator):
         list_callbacks.begin_train(self, dataset, state, control, None)
         try:
             for n_epoch in range(nb_epoch):
+                if is_dist_avail_and_initialized():
+                    dataset.sampler.set_epoch(n_epoch)
+                    if eval_dataset:
+                        eval_dataset.sampler.set_epoch(n_epoch)
+                    torch.distributed.barrier()
                 state.n_epoch = n_epoch
                 loss_epoch = 0
                 list_callbacks.begin_epoch(
@@ -856,6 +866,8 @@ class BaseEnvironnement(BaseEstimator):
                         eval_dataset and
                         self._mod(state.n_iter, iter_eval) == 0
                     ):
+                        if is_dist_avail_and_initialized():
+                            torch.distributed.barrier()
                         self._iter_eval(
                             eval_dataset=eval_dataset,
                             loss_fn=loss_fn,
