@@ -12,7 +12,7 @@ import os
 import sys
 import pickle as pkl
 from abc import abstractmethod
-from copy import deepcopy
+from copy import deepcopy, copy
 from typing import List, Dict, Optional, Callable, Any, NewType
 from dataclasses import dataclass, field
 
@@ -21,6 +21,7 @@ from torch import Tensor, device as torch_device
 from torch.cuda.amp import GradScaler
 from tqdm import tqdm
 
+from dstk.pytorch import is_main_process
 from dstk.utils.meta_interface import SuperclassDocstring
 from dstk.utils.errors import TrainError
 
@@ -677,12 +678,13 @@ class PrintCallback(CallbackInterface):
         res_loss: float,
         **kwargs
     ) -> None:
-        print(
-            f"\nid : {state.id_process}\n"
-            f"Epoch : {state.n_epoch+1}/{state.nb_epoch}, "
-            f"{state.loss_name} : {res_loss:.4f}\n",
-            flush=True
-        )
+        if is_main_process():
+            print(
+                f"\nid : {state.id_process}\n"
+                f"Epoch : {state.n_epoch+1}/{state.nb_epoch}, "
+                f"{state.loss_name} : {res_loss:.4f}\n",
+                flush=True
+            )
 
     def error_interrupt(
         self,
@@ -726,7 +728,7 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
     nb_backup: Optional[int] = None
 
     def __post_init__(self):
-        if not os.path.exists(self.path_backup):
+        if is_main_process() and not os.path.exists(self.path_backup):
             os.makedirs(self.path_backup)
 
     def end_step(
@@ -738,7 +740,7 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if state.n_iter % self.n_iter == 0:
+        if is_main_process() and state.n_iter % self.n_iter == 0:
             pkl.dump(
                 [
                     model.state_dict(),
@@ -751,7 +753,7 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
                     "wb"
                 )
             )
-        if self.nb_backup:
+        if is_main_process() and self.nb_backup:
             files = sorted(
                 list(
                     filter(
@@ -777,7 +779,7 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.erase_folder:
+        if is_main_process() and self.erase_folder:
             files = list(
                 filter(
                     lambda x: (
@@ -800,7 +802,11 @@ class Backup(CallbackInterface, metaclass=SuperclassDocstring):
         exception: Exception,
         **kwargs
     ) -> None:
-        if isinstance(exception, KeyboardInterrupt) and self.erase_folder:
+        if (
+            is_main_process()
+            and isinstance(exception, KeyboardInterrupt)
+            and self.erase_folder
+        ):
             files = list(
                 filter(
                     lambda x: (
@@ -852,7 +858,7 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
         self.best_epoch: int = 0
         self.best_iter: int = 0
         self.state_dict = None
-        if self.path_early_stopping:
+        if is_main_process() and self.path_early_stopping:
             if not os.path.exists(self.path_early_stopping):
                 os.makedirs(self.path_early_stopping)
 
@@ -870,7 +876,7 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
             if self.minimize else 
             res_loss > self.old_res_loss
         ):
-            if self.verbose:
+            if is_main_process() and self.verbose:
                 print(
                     "\nGain de performance époque/itération : "
                     f"{state.n_epoch+1}/{state.n_iter+1} ;\n"
@@ -884,13 +890,13 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
             self.best_iter = state.n_iter
             if self.path_early_stopping is None:
                 self.state_dict = model._extract_state(model)
-            else:
+            elif is_main_process():
                 model.save_model(
                     f"{self.path_early_stopping}/{state.id_process}.model"
                 )
         elif (
-            self.early_stopping_rounds is not None and 
-            (
+            self.early_stopping_rounds is not None
+            and (
                 res_loss >= self.old_res_loss 
                 if self.minimize else
                 res_loss <= self.old_res_loss
@@ -909,7 +915,7 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.verbose:
+        if is_main_process() and self.verbose:
             print(
                 f"\nMeilleure époque/iteration : {self.best_epoch+1}/"
                 f"{self.best_iter+1}\n"
@@ -925,7 +931,10 @@ class EarlyStoppingCallback(CallbackInterface, metaclass=SuperclassDocstring):
             model.load_weights(
                 f"{self.path_early_stopping}/{state.id_process}.model"
             )
-            os.remove(f"{self.path_early_stopping}/{state.id_process}.model")
+            if is_main_process():
+                os.remove(
+                    f"{self.path_early_stopping}/{state.id_process}.model"
+                )
 
 
 @dataclass
@@ -993,7 +1002,7 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
                 "être appliqué durant la phase d'entrainement, mais seulement "
                 "durant la phase d'évaluation."
             )
-        if self.path_backup:
+        if is_main_process() and self.path_backup:
             if not os.path.exists(self.path_backup):
                 os.makedirs(self.path_backup)
 
@@ -1011,12 +1020,13 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
             **self.scheduler_kwargs
         )
         if self.state_dict:
-            print(
-                "Chargement de paramètres du Learning Rate Scheduler :"
-            )
-            print(self.state_dict)
+            if is_main_process():
+                print(
+                    "Chargement de paramètres du Learning Rate Scheduler :"
+                )
+                print(self.state_dict)
             self.scheduler.load_state_dict(self.state_dict)
-        if self.verbose:
+        if is_main_process() and self.verbose:
             self.old_lr = model.optimizer.state_dict()['param_groups'][0]['lr']
             print(
                 f"A l'initialisation le lr = {self.old_lr}\n",
@@ -1037,7 +1047,7 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
                 self.scheduler.step(res_loss)
             else:
                 self.scheduler.step()
-            if self.verbose:
+            if is_main_process() and self.verbose:
                 lr = model.optimizer.state_dict()['param_groups'][0]['lr']
                 if self.old_lr != lr:
                     print(
@@ -1062,7 +1072,7 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
                 self.scheduler.step(res_loss)
             else:
                 self.scheduler.step()
-            if self.verbose:
+            if is_main_process() and self.verbose:
                 lr = model.optimizer.state_dict()['param_groups'][0]['lr']
                 if self.old_lr != lr:
                     print(
@@ -1072,12 +1082,13 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
                         flush=True
                     )
                     self.old_lr = lr
-        if self.path_backup:
+        if is_main_process() and self.path_backup:
             if state.n_iter % self.n_iter == 0:
                 pkl.dump(
                     self.scheduler.state_dict(),
                     open(
-                        f"{self.path_backup}/{state.id_process}_{state.n_iter}.lrs",
+                        f"{self.path_backup}/{state.id_process}_"
+                        f"{state.n_iter}.lrs",
                         "wb"
                     )
                 )
@@ -1107,7 +1118,7 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.path_backup:
+        if is_main_process() and self.path_backup:
             if self.erase_folder:
                 files = list(
                     filter(
@@ -1131,7 +1142,7 @@ class LRSchedulerCallback(CallbackInterface, metaclass=SuperclassDocstring):
         exception: Exception,
         **kwargs
     ) -> None:
-        if self.path_backup:
+        if is_main_process() and self.path_backup:
             if isinstance(exception, KeyboardInterrupt) and self.erase_folder:
                 files = list(
                     filter(
@@ -1175,7 +1186,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.bar_epoch:
+        if is_main_process() and self.bar_epoch:
             self.epoch_bar = tqdm(
                 total=state.epoch_max_steps,
                 desc=(
@@ -1197,7 +1208,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.bar_epoch:
+        if is_main_process() and self.bar_epoch:
             self.epoch_bar.refresh()
             self.epoch_bar.close()
 
@@ -1210,7 +1221,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.bar_epoch:
+        if is_main_process() and self.bar_epoch:
             self.epoch_bar.update(1)
 
     def begin_eval(
@@ -1222,7 +1233,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: None,
         **kwargs
     ) -> None:
-        if self.bar_eval:
+        if is_main_process() and self.bar_eval:
             self.eval_bar = tqdm(
                 total=state.eval_max_steps,
                 desc="Evaluation",
@@ -1238,7 +1249,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.bar_eval:
+        if is_main_process() and self.bar_eval:
             self.epoch_bar.refresh()
             self.eval_bar.close()
 
@@ -1251,7 +1262,7 @@ class ProgressBarCallback(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        if self.bar_eval:
+        if is_main_process() and self.bar_eval:
             self.eval_bar.update(1)
 
 
@@ -1290,11 +1301,12 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
         flush_secs: int=120,
         filename_suffix: str=''
     ):
-        try:
-            global SummaryWriter
-            from torch.utils.tensorboard import SummaryWriter
-        except ModuleNotFoundError:
-            raise ModuleNotFoundError(
+        if is_main_process():
+            try:
+                global SummaryWriter
+                from torch.utils.tensorboard import SummaryWriter
+            except ModuleNotFoundError:
+                raise ModuleNotFoundError(
                     "Le module TensorBoard n'est pas installé : "
                     "l'instruction  torch.utils.tensorboard renvoit une "
                     "erreur ModuleNotFoundError. Visitez la page du projet "
@@ -1303,14 +1315,23 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
                     "PyTorch. Ou installez directement TensorBoard via pip "
                     "ou conda."
                 )
-        self.writer = SummaryWriter(
-            log_dir=log_dir,
-            comment=comment,
-            purge_step=purge_step,
-            max_queue=max_queue,
-            flush_secs=flush_secs,
-            filename_suffix=filename_suffix
-        )
+            self.writer = SummaryWriter(
+                log_dir=log_dir,
+                comment=comment,
+                purge_step=purge_step,
+                max_queue=max_queue,
+                flush_secs=flush_secs,
+                filename_suffix=filename_suffix
+            )
+
+    def copy(self):
+        r"""
+        Créer une copie de l'objet.
+        Surchage du copy de CallbackInterface car dans le SummaryWriter
+        il y a des éléments non picklelisable (donc non compatible avec
+        le deepcopy de la classe CallbackInterface).
+        """
+        return copy(self)
 
     def end_step(
         self,
@@ -1321,16 +1342,17 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        self.writer.add_scalar(
-            'Loss/train',
-            res_loss,
-            state.n_iter
-        )
-        self.writer.add_scalar(
-            'LR/lr',
-            model.optimizer.param_groups[0]['lr'],
-            state.n_iter
-        )
+        if is_main_process():
+            self.writer.add_scalar(
+                'Loss/train',
+                res_loss,
+                state.n_iter
+            )
+            self.writer.add_scalar(
+                'LR/lr',
+                model.optimizer.param_groups[0]['lr'],
+                state.n_iter
+            )
 
     def end_eval(
         self,
@@ -1341,7 +1363,8 @@ class Supervision(CallbackInterface, metaclass=SuperclassDocstring):
         res_loss: float,
         **kwargs
     ) -> None:
-        self.writer.add_scalar('Loss/eval', res_loss, state.n_iter)
+        if is_main_process():
+            self.writer.add_scalar('Loss/eval', res_loss, state.n_iter)
 
 
 DEFAULT_CALLBACKS = [
