@@ -223,7 +223,7 @@ class BaseClassifierOnline(BaseClassifier):
             y=y,
             eval_dataset=None,
             nb_epoch=1,
-            kargs_dataloader=dict(
+            dataloader_kargs=dict(
                 batch_size=1,
                 shuffle=False,
                 drop_last=False
